library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;

entity cola2 is
 port(
  led: out std_logic;
  data_input_c: in std_logic_vector(31 downto 0);
  data_input_b: in std_logic_vector(31 downto 0);
  data_input_a: in std_logic_vector(31 downto 0);
  aa_dir: out std_logic;
  ab_dir: out std_logic;
  ac_dir: out std_logic;
  ad_dir: out std_logic;
  ba_dir: out std_logic;
  bb_dir: out std_logic;
  bc_dir: out std_logic;
  bd_dir: out std_logic;
  ca_dir: out std_logic;
  cb_dir: out std_logic;
  cc_dir: out std_logic;
  cd_dir: out std_logic;
  ft601_clk: in std_logic;
  ft601_data: inout std_logic_vector(31 downto 0);
  ft601_be: inout std_logic_vector(3 downto 0);
  ft601_rd: out std_logic;
  ft601_wr: out std_logic;
  ft601_txe: in std_logic;
  ft601_rxf: in std_logic;
  ft601_oe: out std_logic;
  ft601_wakeup: in std_logic;
  ft601_siwu: in std_logic;
  ft601_reset: out std_logic
  --switch: in std_logic_vector(9 downto 0)
 );
end entity cola2;

architecture behavioural of cola2 is

 attribute chip_pin: string;
 attribute chip_pin of ft601_clk: signal is ("e10");
 attribute chip_pin of ft601_rxf: signal is ("e16");
 attribute chip_pin of ft601_txe: signal is ("g17");
 attribute chip_pin of ft601_rd: signal is ("a15");
 attribute chip_pin of ft601_wr: signal is ("g16");
 attribute chip_pin of ft601_oe: signal is ("b16");
 attribute chip_pin of ft601_be: signal is ("f15, c15, e15, e14");
 attribute chip_pin of ft601_wakeup: signal is ("d17");
 attribute chip_pin of ft601_reset: signal is ("c16"); 
 attribute chip_pin of ft601_siwu: signal is ("h18"); 
 attribute chip_pin of ft601_data: signal is ("b15, a14, c13, a13, g15, f14, f12, f10, b13, a12, b12, b11, d13, f13, d12, e12, c11, e9, c9, d9, c8, e7, d7, d6, a9, a8, b7, a7, c6, b6, a5, b5");
 --attribute chip_pin of switch: signal is ("ae19, y11, ac10, v10, ab10, w11, ac8, ad13, ae10, ac9");
 attribute chip_pin of data_input_c: signal is ("l22, m22, m21, n21, p22, r22, r21, t22, v21, w22, w21, y22, y21, aa22, ab22, ab21, y20, ab20, aa20, aa19, ab18, aa18, ab17, aa17, aa15, aa14, ab13, aa13, ab12, aa12, ab11, ab10");
 attribute chip_pin of data_input_b: signal is ("k19, l17, l19, l18, j19, k20, k22, k21, v16, y17, y16, w16, y15, v13, y14, u11, v10, y11, u10, v9, y9, t9, w9, t8, aa10, aa9, ab8, aa8, ab7, aa7, ab6, ab5");
 attribute chip_pin of data_input_a: signal is ("t19, t20, u20, v20, v19, v18, y19, w19, p17, n19, n20, p18, p19, t18, r16, r17, u13, u15, v15, v14, t12, u12, r11, r10, u8, u7, u6, v6, r6, t7, r7, p6");
 attribute chip_pin of aa_dir: signal is ("p8");
 attribute chip_pin of ab_dir: signal is ("t10");
 attribute chip_pin of ac_dir: signal is ("m18");
 attribute chip_pin of ad_dir: signal is ("j18");
 attribute chip_pin of ba_dir: signal is ("r9");
 attribute chip_pin of bb_dir: signal is ("y10");
 attribute chip_pin of bc_dir: signal is ("m20");
 attribute chip_pin of bd_dir: signal is ("k17");
 attribute chip_pin of ca_dir: signal is ("n8");
 attribute chip_pin of cb_dir: signal is ("ab15");
 attribute chip_pin of cc_dir: signal is ("u21");
 attribute chip_pin of cd_dir: signal is ("u22");
 attribute chip_pin of led: signal is ("g18");


 component input_engine is
  port(
   buffer_overflow: out std_logic;
   clock: in std_logic;
   data_input: in std_logic_vector(95 downto 0);
   output: out std_logic_vector(31 downto 0);
   rdreq: in std_logic;
   usedw: out std_logic_vector(15 downto 0);
   fifo_clock: in std_logic;
   address: in std_logic_vector(15 downto 0);
   data: in std_logic_vector(15 downto 0);
   wren: in std_logic
  );
 end component input_engine;
 
 component ft601_interface is
  port(
   in_data: in std_logic_vector(31 downto 0);
   out_data: out std_logic_vector(31 downto 0);
   out_wr: out std_logic;
   out_usedw: in std_logic_vector(9 downto 0);
   usb_buffer_overflow: out std_logic := '0';
   in_rden: out std_logic;
   in_usedw: in std_logic_vector(15 downto 0);
   in_clk: in std_logic;
   out_rden: in std_logic;
   usb_clk: in std_logic;
   usb_rd: out std_logic := '1';
   usb_wr: out std_logic := '1';
   usb_txe: in std_logic;
   usb_rxf: in std_logic;
   usb_oe: out std_logic := '1';
   usb_be: inout std_logic_vector(3 downto 0) := (others => 'Z');
   usb_data: inout std_logic_vector(31 downto 0) := (others => 'Z');
   usb_wakeup: in std_logic;
   usb_siwu: in std_logic;
   usb_reset: out std_logic 
  );
 end component ft601_interface;
 
 component pll is
  port(
   refclk: in std_logic;
   rst: in std_logic;
   outclk_0: out std_logic
  );
 end component pll;
 
 signal ft601_input_data: std_logic_vector(31 downto 0);
 signal ft601_output_data: std_logic_vector(31 downto 0);
 signal ft601_in_rden: std_logic;
 signal ft601_in_usedw: std_logic_vector(15 downto 0);
 signal mux_data: std_logic_vector(31 downto 0);
 signal data_input: std_logic_vector(95 downto 0);
 signal pll_out: std_logic := '0';
 signal pll_divider: std_logic_vector(23 downto 0) := "111111111111111111111111";
 signal pll_counter: std_logic_vector(23 downto 0) := "000000000000000000000000";
 signal led_counter: std_logic_vector(28 downto 0) := "00000000000000000000000000000";
 signal int_clk: std_logic := '0';
 signal buffer_overflow: std_logic := '0';
 type speed_mode_type is (disabled, single, double, quad);
 signal speed_mode: speed_mode_type := disabled;
 type trigger_mode_type is (continuous, mode_trigger, mode_range, mode_map);
 signal trigger_mode: trigger_mode_type := continuous;
 signal wren: std_logic := '0';
 
begin

 data_input <= data_input_c & data_input_b & data_input_a;
 aa_dir <= '0';
 ab_dir <= '0';
 ac_dir <= '0';
 ad_dir <= '0';
 ba_dir <= '0';
 bb_dir <= '0';
 bc_dir <= '0';
 bd_dir <= '0';
 ca_dir <= '0';
 cb_dir <= '0';
 cc_dir <= '0';
 cd_dir <= '0';

 usb_interface: ft601_interface
 port map(
  in_data => ft601_input_data,
  out_data => mux_data,
  out_wr => wren,
  out_usedw => "0000000000",
  usb_buffer_overflow => open,
  in_rden => ft601_in_rden,
  in_usedw => ft601_in_usedw,
  in_clk => ft601_clk,
  out_rden => '0',
  usb_clk => ft601_clk,
  usb_rd => ft601_rd,
  usb_wr => ft601_wr,
  usb_txe => ft601_txe,
  usb_rxf => ft601_rxf,
  usb_oe => ft601_oe,
  usb_be => ft601_be,
  usb_data => ft601_data,
  usb_wakeup => ft601_wakeup,
  usb_siwu => ft601_siwu,
  usb_reset => ft601_reset
 );
 
 input_packer: input_engine
 port map(
  buffer_overflow => buffer_overflow,
  clock => int_clk,
  data_input => data_input,
  output => ft601_input_data,
  rdreq => ft601_in_rden,
  usedw => ft601_in_usedw,
  fifo_clock => ft601_clk,
  address => mux_data(31 downto 16),
  data => mux_data(15 downto 0),
  wren => wren
 );
 
 int_clk_pll: pll
 port map(
  refclk => ft601_clk,
  rst => '0',
  outclk_0 => pll_out
 );
 
 int_clk_gen: process(ft601_clk)
 begin
  if ft601_clk'event and ft601_clk = '1' then
   led_counter <= led_counter +1;
   if pll_counter = 0 then
    int_clk <= '1';
   else
    int_clk <= '0';
   end if;
   if pll_counter >= pll_divider then
    pll_counter <= (others => '0');
   else
    pll_counter <= pll_counter + 1;
   end if;
  end if;
 end process int_clk_gen;
 
 led_generator: process(ft601_clk)
 begin
  if ft601_clk'event and ft601_clk = '1' then
   if speed_mode = disabled then
    if led_counter(25) = '1' and led_counter(23) = '0' and led_counter(22) = '1' then
     led <= '1';
    else
     led <= '0';
    end if;
   else
    if buffer_overflow = '1' then
     if led_counter(22) = '1' and led_counter(21) = '1' then
      led <= '0';
     else
      led <= '1';
     end if;
    else
     if led_counter(22) = '1' and led_counter(21) = '1' and led_counter(20) = '1' then
      led <= '1';
     else
      led <= '0';
     end if;
    end if;
   end if;
  end if;
 end process led_generator;
 
 clock_mode_selector: process(ft601_clk)
 begin
  if ft601_clk'event and ft601_clk = '1' then
   if mux_data(31 downto 16) = "1111110111111111" and wren = '1' then
    pll_divider(23 downto 16) <= mux_data(7 downto 0);
   end if;
   if mux_data(31 downto 16) = "1111110111111110" and wren = '1' then
    pll_divider(15 downto 0) <= mux_data(15 downto 0);
   end if;
   if mux_data(31 downto 16) = "1111111011111111" and wren = '1' then
    case mux_data(3 downto 0) is
     when "0000" =>
      speed_mode <= disabled;
     when "0001" =>
      speed_mode <= single;
     when "0010" =>
      speed_mode <= double;
     when "0011" =>
      speed_mode <= quad;
     when others =>
      speed_mode <= disabled;
    end case;
    case mux_data(7 downto 4) is
     when "0000" =>
      trigger_mode <= continuous;
     when "0001" =>
      trigger_mode <= mode_trigger;
     when "0010" =>
      trigger_mode <= mode_range;
     when "0011" =>
      trigger_mode <= mode_map;
     when others =>
      trigger_mode <= continuous;
    end case;
   end if;   
  end if;
 end process clock_mode_selector; 
 
end behavioural;

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;

entity test_bench is
end entity test_bench;

architecture behavioural of test_bench is

 component cola2 is
  port(
   --data_input: in std_logic_vector(95 downto 0);
   ft601_clk: in std_logic;
   ft601_data: inout std_logic_vector(31 downto 0);
   ft601_be: inout std_logic_vector(3 downto 0);
   ft601_rd: out std_logic;
   ft601_wr: out std_logic;
   ft601_txe: in std_logic;
   ft601_rxf: in std_logic;
   ft601_oe: out std_logic;
   ft601_wakeup: in std_logic;
   ft601_siwu: in std_logic;
   ft601_reset: out std_logic;
   switch: in std_logic_vector(9 downto 0)
  );
 end component cola2;

 signal ft601_clk: std_logic;
 signal ft601_data: std_logic_vector(31 downto 0);
 signal ft601_be: std_logic_vector(3 downto 0);
 signal ft601_rd: std_logic;
 signal ft601_wr: std_logic;
 signal ft601_txe: std_logic;
 signal ft601_rxf: std_logic;
 signal ft601_oe: std_logic;
 signal ft601_wakeup: std_logic;
 signal ft601_siwu: std_logic;
 signal ft601_reset: std_logic;
 signal switch: std_logic_vector(9 downto 0);
 
 constant ft601_clk_period: time := 10 ns;
 
begin

 uut: cola2
 port map(
  ft601_clk => ft601_clk,
  ft601_data => ft601_data,
  ft601_be => ft601_be,
  ft601_rd => ft601_rd,
  ft601_wr => ft601_wr,
  ft601_txe => ft601_txe,
  ft601_rxf => ft601_rxf,
  ft601_oe => ft601_oe,
  ft601_wakeup => ft601_wakeup,
  ft601_siwu => ft601_siwu,
  ft601_reset => ft601_reset,
  switch => switch
 );

 ft601_clk_generate: process
 begin
  ft601_clk <= '1';
  wait for ft601_clk_period/2;
  ft601_clk <= '0';
  wait for ft601_clk_period/2;
 end process ft601_clk_generate;

 stimulus: process
  --variable seed1, seed2: positive;
  --variable rand: real; 
 begin
  ft601_txe <= '0';
  switch <= "0000000000";
  wait;
 end process stimulus;
 
end behavioural;