library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;
use ieee.numeric_std.all;
use ieee.math_real.all;

library altera_mf;
use altera_mf.altera_mf_components.all;

entity ft601_interface is
 port(
  in_data: in std_logic_vector(31 downto 0);
  out_data: out std_logic_vector(31 downto 0);
  out_wr: out std_logic;
  out_usedw: in std_logic_vector(9 downto 0);
  usb_buffer_overflow: out std_logic := '0';
  in_rden: out std_logic;
  in_usedw: in std_logic_vector(15 downto 0);
  in_clk: in std_logic;
  out_rden: in std_logic;
  usb_clk: in std_logic;
  usb_rd: out std_logic := '1';
  usb_wr: out std_logic := '1';
  usb_txe: in std_logic;
  usb_rxf: in std_logic;
  usb_oe: out std_logic := '1';
  usb_be: inout std_logic_vector(3 downto 0) := (others => 'Z');
  usb_data: inout std_logic_vector(31 downto 0) := (others => 'Z');
  usb_wakeup: in std_logic;
  usb_siwu: in std_logic;
  usb_reset: out std_logic
 );
end entity ft601_interface;

architecture behavioural of ft601_interface is

 type ft601_fsm is (idle, tx_clear1, tx_clear2, tx_write, tx_term, rx_flag1, rx_flag2, rx_flag3, rx_oe, rx_read); 
 
 signal ft601_state: ft601_fsm;
 
 signal fifo_in_dat: std_logic_vector(31 downto 0) := (others => '0');
 signal fifo_in_rd: std_logic := '0';
 signal fifo_in_be: std_logic_vector(3 downto 0) := (others => '0');
 signal fifo_in_counter: std_logic_vector(9 downto 0) := (others => '0');
 signal fifo_out_dat: std_logic_vector(31 downto 0) := (others => '0');
 signal fifo_out_dat_buffered: std_logic_vector(31 downto 0) := (others => '0'); 
 signal fifo_out_wr: std_logic := '0';
 signal fifo_out_be: std_logic_vector(3 downto 0) := (others => '0');
 signal fifo_out_usedw: std_logic_vector(9 downto 0);
 
begin

 usb_reset <= '1';

 ft601_handle: process(usb_clk)
 begin
  if usb_clk'event and usb_clk = '1' then
   out_data <= fifo_out_dat;
   out_wr <= fifo_out_wr;
   case ft601_state is
    when idle =>
     usb_oe <= '1';
     usb_rd <= '1';
     usb_wr <= '1';
     usb_data <= (others => 'Z');
     usb_be <= (others => 'Z');     
     if usb_txe = '0' and in_usedw > "0000000000000000" then
      ft601_state <= tx_clear1;
     end if;
     if usb_rxf = '0' and out_usedw(out_usedw'left-1) = '0' then
      ft601_state <= rx_flag1;
     end if;
    when tx_clear1 =>
     ft601_state <= tx_clear2;
     in_rden <= '1';
    when tx_clear2 =>
     ft601_state <= tx_write;
     in_rden <= '1';
     usb_wr <= '0'; 
     usb_data <= in_data;
     usb_be <= "1111";
    when tx_write =>
     if usb_txe = '1' or in_usedw < "0000000000000001" then
      in_rden <= '0';
      ft601_state <= idle;
      usb_data <= in_data;
      usb_be <= "1111";
      fifo_in_counter <= (others => '0');
     else
      if fifo_in_counter = "1111111111" then
       usb_data <= (others => 'Z');
       usb_be <= (others => 'Z');
       ft601_state <= tx_term;
       fifo_in_counter <= (others => '0');
      else
       if fifo_in_counter = "1111111110" then
        in_rden <= '0';
        usb_data <= in_data;
       else
        in_rden <= '1';
        ft601_state <= tx_write;
        fifo_in_counter <= fifo_in_counter +1;
        usb_data <= in_data;
        usb_be <= "1111";
       end if;
      end if;
     end if;
    when tx_term =>
     usb_wr <= '1';
     ft601_state <= idle;
    when rx_flag1 =>
     ft601_state <= rx_flag2;
    when rx_flag2 =>
     ft601_state <= rx_flag3;
    when rx_flag3 =>
     usb_oe <= '0';
     ft601_state <= rx_oe;
    when rx_oe =>
     usb_rd <= '0';
     ft601_state <= rx_read;
     fifo_out_dat <= usb_data;
    when rx_read =>
     if usb_rxf = '1' then
      ft601_state <= idle;
      fifo_out_wr <= '0';
     else
      ft601_state <= rx_read;
      fifo_out_dat <= usb_data;
      fifo_out_wr <= '1';
     end if;
    when others =>
   end case;
  end if;
 end process ft601_handle;

end behavioural;