library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;

entity mux_4to1 is
 port(
  input: in std_logic_vector(3 downto 0);
  output: out std_logic;
  address: in std_logic_vector(1 downto 0);
  clock: in std_logic
 );
end entity mux_4to1;

architecture behavioural of mux_4to1 is
begin
 multiplexer: process(clock)
 begin
  if clock'event and clock = '1' then
   case address is
    when "00" =>
     output <= input(0);
    when "01" =>
     output <= input(1);
    when "10" =>
     output <= input(2);
    when "11" =>
     output <= input(3);
    when others =>
     output <= '0';
   end case;
  end if;
 end process multiplexer;
end behavioural;

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;

entity mux_128to1 is
 port(
  input: in std_logic_vector(127 downto 0);
  output: out std_logic;
  address: in std_logic_vector(6 downto 0);
  clock: in std_logic
 );
end entity mux_128to1;

architecture behavioural of mux_128to1 is

 component mux_4to1 is
  port(
   input: in std_logic_vector(3 downto 0);
   output: out std_logic;
   address: in std_logic_vector(1 downto 0);
   clock: in std_logic
  ); 
 end component mux_4to1;

 type address_type is array(31 downto 0) of std_logic_vector(1 downto 0);
 
 signal inter_1: std_logic_vector(31 downto 0);
 signal inter_2: std_logic_vector(7 downto 0);
 signal inter_3: std_logic_vector(3 downto 0);
 signal int_address: std_logic_vector(1 downto 0);
 
begin

 int_address <= '0' & address(6);

 stage_1: for i in 0 to 31 generate
  stage_1_mux: mux_4to1
  port map(
   input => input((i*4)+3 downto (i*4)),
   output => inter_1(i),
   address => address(1 downto 0),
   clock => clock
  );
 end generate;
 
 stage_2: for i in 0 to 7 generate
  stage_2_mux: mux_4to1
  port map(
   input => inter_1((i*4)+3 downto (i*4)),
   output => inter_2(i),
   address => address(3 downto 2),
   clock => clock
  );
 end generate; 

 stage_3: for i in 0 to 1 generate
  stage_3_mux: mux_4to1
  port map(
   input => inter_2((i*4)+3 downto (i*4)),
   output => inter_3(i),
   address => address(5 downto 4),
   clock => clock
  );
 end generate; 
 
 stage_4_mux: mux_4to1
 port map(
  input => inter_3,
  output => output,
  address => int_address,
  clock => clock
 );
 
end behavioural;

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;

entity mux128x128 is
 port(
  input: in std_logic_vector(127 downto 0);
  output: out std_logic_vector(127 downto 0);
  address: in std_logic_vector(6 downto 0);
  data: in std_logic_vector(6 downto 0);
  wren : in std_logic;
  clock: in std_logic
 );
end entity mux128x128;

architecture behavioural of mux128x128 is

 component mux_128to1 is
  port(
   input: in std_logic_vector(127 downto 0);
   output: out std_logic;
   address: in std_logic_vector(6 downto 0);
   clock: in std_logic
  );
 end component mux_128to1;

 type address_array is array(127 downto 0) of std_logic_vector(6 downto 0);
 
 signal int_address: address_array;
 
begin

 address_fill: for i in 0 to 127 generate
  address_latch: process(clock)
  begin
   if clock'event and clock = '1' then
    if wren = '1' and address = i then
     int_address(i) <= data;
    end if;
   end if;
  end process address_latch;
  
  multiplexer: mux_128to1
  port map(
   input => input,
   output => output(i),
   address => int_address(i),
   clock => clock
  );
  
 end generate;

end behavioural;