library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;

entity input_engine is
 port(
  buffer_overflow: out std_logic;
  clock: in std_logic;
  data_input: in std_logic_vector(95 downto 0);
  output: out std_logic_vector(31 downto 0);
  rdreq: in std_logic;
  usedw: out std_logic_vector(15 downto 0);
  fifo_clock: in std_logic;
  address: in std_logic_vector(15 downto 0);
  data: in std_logic_vector(15 downto 0);
  wren: in std_logic
 );
end entity input_engine;

architecture behavioural of input_engine is

 component mux128x128 is
  port(
   input: in std_logic_vector(127 downto 0);
   output: out std_logic_vector(127 downto 0);
   address: in std_logic_vector(6 downto 0);
   data: in std_logic_vector(6 downto 0);
   wren: in std_logic;
   clock: in std_logic
  );
 end component mux128x128;
 
 component trigger_engine is
  port(
   input: in std_logic_vector(127 downto 0);
   trigger: out std_logic;
   address: in std_logic_vector(15 downto 0);
   data: in std_logic_vector(15 downto 0);
   wren: in std_logic;
   clock_100: in std_logic;
   clock: in std_logic
  ); 
 end component trigger_engine;
 
 component bulkmem is
  port(
   clock: in std_logic;
	  data: in std_logic_vector(31 downto 0);
	  rdreq: in std_logic;
	  wrreq: in std_logic;
	  usedw: out std_logic_vector(15 downto 0);
	  q: out std_logic_vector(31 downto 0)
  );
 end component bulkmem;
 
 type raw_packer_state_type is (idle, pack_3, pack_2, pack_1);

 signal data_input_muxed: std_logic_vector(127 downto 0);
 signal data_input_muxed_delay: std_logic_vector(127 downto 0);
 signal old_data_input: std_logic_vector(95 downto 0);
 signal data_input_buffer: std_logic_vector(95 downto 0);
 signal raw_packer_state: raw_packer_state_type := idle;
 type speed_mode_type is (disabled, single, double, quad);
 signal speed_mode: speed_mode_type := disabled;
 type trigger_mode_type is (continuous, mode_trigger, mode_range, mode_map);
 signal trigger_mode: trigger_mode_type := continuous;
 signal compression_counter: std_logic_vector(6 downto 0) := "0000000";
 signal fifo_data_in: std_logic_vector(31 downto 0);
 signal fifo_write: std_logic;
 --signal clock_shift: std_logic_vector(1 downto 0);
 signal data_input_inter: std_logic_vector(127 downto 0);
 signal int_usedw: std_logic_vector(15 downto 0);
 signal int_usedw_buffered: std_logic_vector(15 downto 0);
 signal int_buffer_overflow: std_logic := '0';
 signal int_trigger: std_logic;
 signal mux_wren: std_logic;
 
begin
 
 data_input_inter <= "00000000000000000000000000000000" & data_input;
 buffer_overflow <= int_buffer_overflow;
 mux_wren <= wren when address(15 downto 8) = "11111111" else '0';

 input_mux: mux128x128
 port map(
  input => data_input_inter,
  output => data_input_muxed,
  address => address(6 downto 0),
  data => data(6 downto 0),
  wren => mux_wren,
  clock => fifo_clock
 );

 trigger: trigger_engine
 port map(
  input => data_input_muxed,
  trigger => int_trigger,
  address => address,
  data => data,
  wren => wren,
  clock_100 => fifo_clock,
  clock => clock
 );

 mode_selector: process(fifo_clock)
 begin
  if fifo_clock'event and fifo_clock = '1' then
   if address = "1111111011111111" and wren = '1' then
    case data(3 downto 0) is
     when "0000" =>
      speed_mode <= disabled;
     when "0001" =>
      speed_mode <= single;
     when "0010" =>
      speed_mode <= double;
     when "0011" =>
      speed_mode <= quad;
     when others =>
      speed_mode <= disabled;
    end case;
    case data(7 downto 4) is
     when "0000" =>
      trigger_mode <= continuous;
     when "0001" =>
      trigger_mode <= mode_trigger;
     when "0010" =>
      trigger_mode <= mode_range;
     when "0011" =>
      trigger_mode <= mode_map;
     when others =>
      trigger_mode <= continuous;
    end case;
   end if;
  end if;
 end process mode_selector;
 
 raw_packer: process(fifo_clock)
 begin
  if fifo_clock'event and fifo_clock = '1' then
   int_usedw_buffered <= int_usedw;
   if int_trigger = '1' or (int_usedw_buffered > 0 and int_usedw = 0) then
    data_input_muxed_delay <= data_input_muxed;
   end if;
   case raw_packer_state is
    when idle =>
     if int_trigger = '1' or (int_usedw_buffered > 0 and int_usedw = 0) then
      data_input_buffer <= data_input_muxed_delay(95 downto 0);
      if int_usedw < "1111111111111100" then
       if data_input_muxed(95 downto 72) = old_data_input(95 downto 72) and speed_mode = single then
        if data_input_muxed(71 downto 48) = old_data_input(71 downto 48)then
         if data_input_muxed(47 downto 24) = old_data_input(47 downto 24) then
          if data_input_muxed(23 downto 0) = old_data_input(23 downto 0) then
           if compression_counter < "1111111" then
            compression_counter <= compression_counter +1;
            fifo_write <= '0';
           else
            fifo_data_in <= '0' & compression_counter & data_input_muxed_delay(23 downto 0);
            compression_counter <= (others => '0');
            fifo_write <= '1';
           end if;
          else
           old_data_input(23 downto 0) <= data_input_muxed_delay(23 downto 0);
           fifo_data_in <= '0' & compression_counter & data_input_muxed_delay(23 downto 0);
           compression_counter <= (others => '0');
           fifo_write <= '1';
          end if;
         else
          fifo_data_in <= "10000000" & data_input_muxed_delay(47 downto 24);
          raw_packer_state <= pack_1;
          fifo_write <= '1';
         end if;
        else
         fifo_data_in <= "10000001" & data_input_muxed_delay(71 downto 48);
         raw_packer_state <= pack_2;
         fifo_write <= '1';
        end if;
       else
        case speed_mode is
         when single =>
          fifo_data_in <= "10000010" & data_input_muxed_delay(95 downto 72);
          raw_packer_state <= pack_3;
          fifo_write <= '1';
         when double =>
          if data_input_muxed(47 downto 24) = old_data_input(47 downto 24) then
           if data_input_muxed(23 downto 0) = old_data_input(23 downto 0) then
            if compression_counter < "1111111" then
             compression_counter <= compression_counter +1;
             fifo_write <= '0';
            else
             fifo_data_in <= '0' & compression_counter & data_input_muxed_delay(23 downto 0);
             compression_counter <= (others => '0');
             fifo_write <= '1';
            end if;
           else
            old_data_input(23 downto 0) <= data_input_muxed_delay(23 downto 0);
            fifo_data_in <= '0' & compression_counter & data_input_muxed_delay(23 downto 0);
            compression_counter <= (others => '0');
            fifo_write <= '1';
           end if;
          else
           fifo_data_in <= "10000000" & data_input_muxed_delay(47 downto 24);
           raw_packer_state <= pack_1;
           fifo_write <= '1';
          end if;
         when quad =>
          if data_input_muxed(23 downto 0) = old_data_input(23 downto 0) then
           if compression_counter < "1111111" then
            compression_counter <= compression_counter +1;
            fifo_write <= '0';
           else
            fifo_data_in <= '0' & compression_counter & data_input_muxed_delay(23 downto 0);
            compression_counter <= (others => '0');
            fifo_write <= '1';
           end if;
          else
           old_data_input(23 downto 0) <= data_input_muxed(23 downto 0);
           fifo_data_in <= '0' & compression_counter & data_input_muxed_delay(23 downto 0);
           compression_counter <= (others => '0');
           fifo_write <= '1';
          end if;
         when others =>
        end case;
       end if;
      else
       int_buffer_overflow <= '1';
      end if;
     else
      fifo_write <= '0';
     end if;
    when pack_3 =>
     fifo_data_in <= "10000001" & data_input_buffer(71 downto 48);
     raw_packer_state <= pack_2;
     fifo_write <= '1';
    when pack_2 =>
     fifo_data_in <= "10000000" & data_input_buffer(47 downto 24);
     raw_packer_state <= pack_1;
     fifo_write <= '1';
    when pack_1 =>
     fifo_data_in <= '0' & compression_counter & data_input_buffer(23 downto 0);
     compression_counter <= (others => '0');
     raw_packer_state <= idle;
     fifo_write <= '1';
     old_data_input <= data_input_buffer;
   end case;
 	end if;
 end process raw_packer;
 
 usedw <= int_usedw;
 
 packer_fifo: bulkmem
 port map(
  clock => fifo_clock,
  data => fifo_data_in,
  rdreq => rdreq,
  wrreq => fifo_write,
  usedw => int_usedw,
  q => output
 );
 
end behavioural;

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;
use ieee.numeric_std.all;
use ieee.math_real.all;

entity input_engine_testbench is
end entity input_engine_testbench;

architecture behavioural of input_engine_testbench is

 component input_engine is
  port(
   clock: in std_logic;
   data_input: in std_logic_vector(95 downto 0);
   output: out std_logic_vector(31 downto 0);
   rdreq: in std_logic;
   fifo_clock: in std_logic
  );
 end component input_engine;

 signal clock: std_logic;
 signal data_input: std_logic_vector(95 downto 0);
 signal output: std_logic_vector(31 downto 0);
 signal rdreq: std_logic;
 signal fifo_clock: std_logic;
 
 constant fifo_clock_period: time := 10 ns;
 constant clock_period: time := 243 ns;
 
begin

 fifo_clock_generate: process
 begin
  fifo_clock <= '1';
  wait for fifo_clock_period/2;
  fifo_clock <= '0';
  wait for fifo_clock_period/2;
 end process fifo_clock_generate;
 
 uut: input_engine
 port map(
  clock => clock,
  data_input => data_input,
  output => output,
  rdreq => rdreq,
  fifo_clock => fifo_clock
 );
 
 stimulus: process
 begin
  data_input <= conv_std_logic_vector(0, 96); 
  wait for clock_period*10;
  data_input <= conv_std_logic_vector(1, 96);
  clock <= '1';
  wait for clock_period/2;
  clock <= '0';
  wait for clock_period/2;
  data_input <= conv_std_logic_vector(2, 96);
  clock <= '1';
  wait for clock_period/2;
  clock <= '0';
  wait for clock_period/2;
  data_input <= conv_std_logic_vector(3, 96);
  clock <= '1';
  wait for clock_period/2;
  clock <= '0';
  wait for clock_period/2;
  data_input <= conv_std_logic_vector(4, 96);
  clock <= '1';
  wait for clock_period/2;
  clock <= '0';
  wait for clock_period/2;
  data_input <= conv_std_logic_vector(0, 96); 
  wait for clock_period*100; 
  wait;
 end process stimulus;
 
end behavioural;