library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;

entity trigger_engine is
 port(
  input: in std_logic_vector(127 downto 0);
  trigger: out std_logic;
  address: in std_logic_vector(15 downto 0);
  data: in std_logic_vector(15 downto 0);
  wren: in std_logic;
  clock_100: in std_logic;
  clock: in std_logic
 );
end entity trigger_engine;

architecture behavioural of trigger_engine is

 type capture_type is array(31 downto 0) of std_logic_vector(127 downto 0);
 signal capture_data: capture_type;
 signal capture_mask: capture_type;
 signal capture_trigger: std_logic_vector(31 downto 0);
 type speed_mode_type is (disabled, single, double, quad);
 signal speed_mode: speed_mode_type := disabled;
 type trigger_mode_type is (continuous, mode_trigger, mode_range, mode_map);
 signal trigger_mode: trigger_mode_type := continuous;
 signal range_mask: std_logic_vector(127 downto 0);
 signal range_lo: std_logic_vector(127 downto 0);
 signal range_hi: std_logic_vector(127 downto 0);
 signal range_trigger: std_logic := '0';

begin

 trigger_concentrator: process(clock_100)
 begin
  if clock_100'event and clock_100 = '1' then
   case(trigger_mode) is
    when mode_trigger =>
     if capture_trigger > 0 then
      trigger <= clock;
     else
      trigger <= '0';
     end if;
    when continuous =>
     trigger <= clock;
    when mode_range =>
     trigger <= range_trigger;
    when others =>
     trigger <= '0';
   end case;
  end if;
 end process trigger_concentrator;
 
 capture_trigger_generator: for i in 0 to 31 generate
  capture_trigger_process: process(clock)
  begin
   if clock'event and clock = '1' then
    if ((input and capture_mask(i)) = (capture_data(i) and capture_mask(i))) then
     capture_trigger(i) <= '1';
    else
     capture_trigger(i) <= '0';
    end if;
   end if;
  end process capture_trigger_process;
  
  capture_data_load: process(clock_100)
  begin
   if clock_100'event and clock_100 = '1' then
    if address(15 downto 8) = "00000000" and address(7 downto 3) = i and wren = '1' then
     case(address(2 downto 0)) is
      when "000" =>
       capture_data(i)(15 downto 0) <= data;
      when "001" =>
       capture_data(i)(31 downto 16) <= data;
      when "010" =>
       capture_data(i)(47 downto 32) <= data;
      when "011" =>
       capture_data(i)(63 downto 48) <= data;
      when "100" =>
       capture_data(i)(79 downto 64) <= data;
      when "101" =>
       capture_data(i)(95 downto 80) <= data;
      when "110" =>
       capture_data(i)(111 downto 96) <= data;
      when "111" =>
       capture_data(i)(127 downto 112) <= data;
      when others =>
     end case;
    end if;
    if address(15 downto 8) = "00000001" and address(7 downto 3) = i and wren = '1' then
     case(address(2 downto 0)) is
      when "000" =>
       capture_mask(i)(15 downto 0) <= data;
      when "001" =>
       capture_mask(i)(31 downto 16) <= data;
      when "010" =>
       capture_mask(i)(47 downto 32) <= data;
      when "011" =>
       capture_mask(i)(63 downto 48) <= data;
      when "100" =>
       capture_mask(i)(79 downto 64) <= data;
      when "101" =>
       capture_mask(i)(95 downto 80) <= data;
      when "110" =>
       capture_mask(i)(111 downto 96) <= data;
      when "111" =>
       capture_mask(i)(127 downto 112) <= data;
      when others =>
     end case;
    end if;
   end if;
  end process capture_data_load;
 end generate;
 
 lo_hi_range_load: process(clock_100)
 begin
  if clock_100'event and clock_100 = '1' then
   if address(15 downto 3) = "0000001000000" and wren = '1' then
    case(address(2 downto 0)) is
     when "000" =>
      range_lo(15 downto 0) <= data;
     when "001" =>
      range_lo(31 downto 16) <= data;
     when "010" =>
      range_lo(47 downto 32) <= data;
     when "011" =>
      range_lo(63 downto 48) <= data;
     when "100" =>
      range_lo(79 downto 64) <= data;
     when "101" =>
      range_lo(95 downto 80) <= data;
     when "110" =>
      range_lo(111 downto 96) <= data;
     when "111" =>
      range_lo(127 downto 112) <= data;
     when others =>
    end case;
   end if;    
   if address(15 downto 3) = "0000001000001" and wren = '1' then
    case(address(2 downto 0)) is
     when "000" =>
      range_hi(15 downto 0) <= data;
     when "001" =>
      range_hi(31 downto 16) <= data;
     when "010" =>
      range_hi(47 downto 32) <= data;
     when "011" =>
      range_hi(63 downto 48) <= data;
     when "100" =>
      range_hi(79 downto 64) <= data;
     when "101" =>
      range_hi(95 downto 80) <= data;
     when "110" =>
      range_hi(111 downto 96) <= data;
     when "111" =>
      range_hi(127 downto 112) <= data;
     when others =>
    end case;
   end if;
   if address(15 downto 3) = "0000001000010" and wren = '1' then
    case(address(2 downto 0)) is
     when "000" =>
      range_mask(15 downto 0) <= data;
     when "001" =>
      range_mask(31 downto 16) <= data;
     when "010" =>
      range_mask(47 downto 32) <= data;
     when "011" =>
      range_mask(63 downto 48) <= data;
     when "100" =>
      range_mask(79 downto 64) <= data;
     when "101" =>
      range_mask(95 downto 80) <= data;
     when "110" =>
      range_mask(111 downto 96) <= data;
     when "111" =>
      range_mask(127 downto 112) <= data;
     when others =>
    end case;
   end if;    
  end if;
 end process lo_hi_range_load;
  
 range_trigger_process: process(clock)
 begin
  if clock'event and clock = '1' then
   if ((input & range_mask) >= (range_lo & range_mask)) and ((input & range_mask) <= (range_hi & range_mask)) then
    range_trigger <= '1';
   else
    range_trigger <= '0';
   end if;
  end if;
 end process range_trigger_process; 

 mode_selector: process(clock_100)
 begin
  if clock_100'event and clock_100 = '1' then
   if address = "1111111011111111" and wren = '1' then
    case data(3 downto 0) is
     when "0000" =>
      speed_mode <= disabled;
     when "0001" =>
      speed_mode <= single;
     when "0010" =>
      speed_mode <= double;
     when "0011" =>
      speed_mode <= quad;
     when others =>
      speed_mode <= disabled;
    end case;
    case data(7 downto 4) is
     when "0000" =>
      trigger_mode <= continuous;
     when "0001" =>
      trigger_mode <= mode_trigger;
     when "0010" =>
      trigger_mode <= mode_range;
     when "0011" =>
      trigger_mode <= mode_map;
     when others =>
      trigger_mode <= continuous;
    end case;
   end if;
  end if;
 end process mode_selector;

end behavioural;